###############################################################################
#                                                                             #
# Player.pm    - Part of the halfStats distribution                           #
#              - Copyright Zach Karpinski                                     #
#              - Distributed under the GPL terms - see the docs for info      #
#              - http://www.halfstats.com                                     #
#                                                                             #
###############################################################################
# CVS $Id: Player.pm,v 1.5 2001/02/03 05:10:10 smccrory Exp $
###############################################################################
use strict;
use Time::Local;

package Player;

#------------------------------------------------------------------------------
# Object constructor for player class
sub new
{
    my $class = shift;
    my $self = {};

    bless $self, $class;

    $self->{'exists'}     = 1;
    $self->{'kills'}      = 0;
    $self->{'deaths'}     = 0;
    $self->{'suicides'}   = 0;
    $self->{'skill'}      = 1000;
    $self->{'total_time'} = 0;
    $self->{'start_time'} = 0;
    $self->{'disconnects'}= 0;
    $self->{'name'}       = "";
    $self->{'name_clean'} = "";
    $self->{'name_html'}  = "";
    $self->{'last_ip'}    = "";
    $self->{'saved_wonId'}= "";
    $self->{'wonId'}      = "";
    $self->{'last_date'}  = 0;
    $self->{'weapons'}    = ();
    $self->{'victims'}    = ();
    $self->{'cstrike'}    = ();
    $self->{'previous_names'} = ();

    $self->{'cstrike'}{'Tjoins'}        = 0;
    $self->{'cstrike'}{'CTjoins'}       = 0;
    $self->{'cstrike'}{'hTouch'}        = 0;
    $self->{'cstrike'}{'hKills'}        = 0;
    $self->{'cstrike'}{'bPlants'}       = 0;
    $self->{'cstrike'}{'bDets'}         = 0;
    $self->{'cstrike'}{'hRescue'}       = 0;
    $self->{'cstrike'}{'bDefuse'}       = 0;
    $self->{'cstrike'}{'vKills'}        = 0;
    return($self);
}

#------------------------------------------------------------------------------
# Object destructor for player class (used mostly for development when trying
#     to track when an object is detroyed and memory freed)
#sub DESTROY
#{
#    my $self = shift;
#    print("$self dying\n");
#}

#------------------------------------------------------------------------------
# Object method to record a kill
sub kill
{
    my $self = shift;
    my $victim = shift;
    my $weapon = shift;
    my $map = shift;

    my $kSkill;
    my $vSkill;
    my $calcSkill;


    if ( $victim && $weapon && $map)
    {
        if ( $self->{'wonId'} ne $victim->{'wonId'} )
        {
            $self->{'kills'} += 1; 
            $victim->{'deaths'} += 1;  # so much for encapsulation ;)
            
            my $vId = $victim->{'wonId'};
            
            $kSkill = $self->{'skill'}; if($kSkill < 1000) { $kSkill = 1000; }
            $vSkill = $victim->{'skill'}; if($vSkill < 1000) { $vSkill = 1000; }
            
            # Calculate base skill level
            $calcSkill = ($vSkill/$kSkill) * 10;
            
            $self->{'skill'} = sprintf("%.1f", ($kSkill + $calcSkill));
            $victim->{'skill'} = sprintf("%.1f", ($vSkill - $calcSkill));
            
            $self->useWeapon($weapon);
            
            $self->{'victims'}{$vId}{'total'} += 1;
            $self->{'victims'}{$vId}{'weapons'}{$weapon} += 1;
            
            if(!$self->{'maps'}{'kills'}{$map}) { $self->{'maps'}{'kills'}{$map} = 1; }
            else { $self->{'maps'}{'kills'}{$map} += 1; }
            
            if(!$victim->{'maps'}{'deaths'}{$map}) { $victim->{'maps'}{'deaths'}{$map} = 1; }
            else { $victim->{'maps'}{'deaths'}{$map} += 1; }
        }
        else
        {
            # SUICIDE!!
            my $kSkill = $self->{'skill'}; 
            if($kSkill < 1000) { $kSkill = 1000; }
            $self->{'skill'} = sprintf("%.1f", ($kSkill - (($kSkill/1000)*5)));
            $self->{'suicides'} += 1;
            if($map && !$self->{'maps'}{'suicides'}{$map}) { $self->{'maps'}{'suicides'}{$map} = 1; }
            elsif($map) { $self->{'maps'}{'suicides'}{$map} += 1; }
        }
    }
}

#------------------------------------------------------------------------------
# Clean up some references and stuff
sub cleanMyself
{
    my ($victim, $weapon, $self, %vhash, %whash);
    $self = shift;

    foreach $victim (keys %{$self->{'victims'}})
    {
	if($victim && $self->{'victims'}{$victim}{'total'})
	{
	    if($self->{'victims'}{$victim}{'total'} >= 1)
	    {
		    $vhash{$victim} = $self->{'victims'}{$victim};
	    }
	}
    }

    undef $self->{'victims'};
    $self->{'victims'} = \%vhash;
   
    foreach $weapon (keys %{$self->{'weapons'}})
    {
    	if($weapon && $self->{'weapons'}{$weapon})
    	{
    	    if($self->{'weapons'}{$weapon} >= 1)
    	    {
    		    $whash{$weapon} = $self->{'weapons'}{$weapon};
    	    }
    	}
    }

    undef $self->{'weapons'};
    $self->{'weapons'} = \%whash;
   
}

#------------------------------------------------------------------------------
# Record use of a weapon
sub useWeapon
{
    my $self = shift;
    my $weapon = shift;
    if(!$self->{'weapons'}{$weapon}) { $self->{'weapons'}{$weapon} = 1; }
    else { $self->{'weapons'}{$weapon} += 1; }
}

#------------------------------------------------------------------------------
# Retrieve kills against a victim
sub getVictimKills
{
    my $self = shift;
    my $victim = shift;
    if(!$self->{'victims'}{$victim}{'total'}) { undef $self->{'victims'}{$victim}; return(0); }
    else { return($self->{'victims'}{$victim}{'total'}); }
}

#------------------------------------------------------------------------------
# Record a VIP kill
sub killVIP
{
    my $self = shift;
    $self->{'cstrike'}{'vKills'} += 1;
    $self->{'skill'} += 10;
}

#------------------------------------------------------------------------------
# Retrieve VIP kills
sub getVIPKills
{
    my $self = shift;
    if(!$self->{'cstrike'}{'vKills'}) { undef $self->{'cstrike'}{'vKills'}; return(0);}
    else { return($self->{'cstrike'}{'vKills'}); }
}

#------------------------------------------------------------------------------
# Record a hostage rescue
sub hRescue
{
    my $self = shift;
    $self->{'cstrike'}{'hRescue'} += 1;
    $self->{'skill'} += 5;
}

#------------------------------------------------------------------------------
# Retrieve number of hostage rescues
sub getHRescue
{
    my $self = shift;
    if(!$self->{'cstrike'}{'hRescue'}) { undef $self->{'cstrike'}{'hRescue'}; return(0); }
    else { return($self->{'cstrike'}{'hRescue'}); }
}

#------------------------------------------------------------------------------
# Record a bomb defuse
sub defBomb
{
    my $self = shift;
    $self->{'cstrike'}{'bDefuse'} += 1;
    $self->{'skill'} += 10;
}

#------------------------------------------------------------------------------
# Retrieve number of bombs defused
sub getBombDefuse
{
    my $self = shift;
    if(!$self->{'cstrike'}{'bDefuse'}) { undef $self->{'cstrike'}{'bDefuse'}; return(0); }
    else { return($self->{'cstrike'}{'bDefuse'}); }
}

#------------------------------------------------------------------------------
# Record a bomb detonation
sub detBomb
{
    my $self = shift;
    $self->{'cstrike'}{'bDets'} += 1;
    $self->{'skill'} += 5;
}

#------------------------------------------------------------------------------
# Retrieve number of bomb detonations
sub getBombDets
{
    my $self = shift;
    if(!$self->{'cstrike'}{'bDets'}) { undef $self->{'cstrike'}{'bDets'}; return(0);}
    else { return($self->{'cstrike'}{'bDets'}); }
}

#------------------------------------------------------------------------------
# Record bomb plant
sub plantBomb
{
    my $self = shift;
    $self->{'cstrike'}{'bPlants'} += 1;
    $self->{'skill'} += 5;
}

#------------------------------------------------------------------------------
# Get number of bomb plants
sub getBombPlants
{
    my $self = shift;
    if(!$self->{'cstrike'}{'bPlants'}) { undef $self->{'cstrike'}{'bPlants'}; return(0); }
    else { return($self->{'cstrike'}{'bPlants'}); }
}

#------------------------------------------------------------------------------
# Record hostage kill
sub killHostage
{
    my $self = shift;
    $self->{'cstrike'}{'hKills'} += 1;
    $self->{'skill'} -= 10;
}

#------------------------------------------------------------------------------
# Retrieve number of hostages killed
sub getHostageKills
{
    my $self = shift;
    if(!$self->{'cstrike'}{'hKills'}) { undef $self->{'cstrike'}{'hKills'}; return(0); }
    else { return($self->{'cstrike'}{'hKills'}); }
}

#------------------------------------------------------------------------------
# Record hostage touch
sub touchHostage
{
    my $self = shift;
    $self->{'cstrike'}{'hTouch'} += 1;
    $self->{'skill'} += 2.5;
}

#------------------------------------------------------------------------------
# Retrieve number of hostage touches
sub getHostageTouches
{
    my $self = shift;
    if(!$self->{'cstrike'}{'hTouch'}) { undef $self->{'cstrike'}{'hTouch'}; return(0);}
    else { return($self->{'cstrike'}{'hTouch'}); }
}

#------------------------------------------------------------------------------
# Record Terrist join
sub joinT
{
    my $self = shift;
    $self->{'cstrike'}{'Tjoins'} += 1;
}

#------------------------------------------------------------------------------
# Record Counter-Terrorist join
sub joinCT
{
    my $self = shift;
    $self->{'cstrike'}{'CTjoins'} += 1;
}

#------------------------------------------------------------------------------
# Retrieve number of Terrorist joins
sub getTJoins
{
    my $self = shift;
    if(!$self->{'cstrike'}{'Tjoins'}) { $self->{'cstrike'}{'Tjoins'} = 0; }
    return($self->{'cstrike'}{'Tjoins'});
}

#------------------------------------------------------------------------------
# Retrieve number of Counter-Terrorist joins
sub getCTJoins
{
    my $self = shift;
    if(!$self->{'cstrike'}{'CTjoins'}) { $self->{'cstrike'}{'CTjoins'} = 0; }
    return($self->{'cstrike'}{'CTjoins'});
}

#------------------------------------------------------------------------------
# Record WON ID
sub setWonId
{
    my $self = shift;
    $self->{'wonId'} = shift;
}

#------------------------------------------------------------------------------
# Save a given WON ID
sub saveWonId
{
    my $self = shift;
    $self->{'saved_wonId'} = shift;
}

#------------------------------------------------------------------------------
# Retrieve WON ID
sub getWonId
{
    my $self = shift;
    return($self->{'wonId'});
}

#------------------------------------------------------------------------------
# Retrieve number of uses of a particular weapon
sub getWeaponUses
{
    my $self = shift;
    my $weapon = shift;
    if(!$self->{'weapons'}{$weapon}) { undef $self->{'weapons'}{$weapon}; return(0);}
    else { return($self->{'weapons'}{$weapon}); }
}

#------------------------------------------------------------------------------
# Retrieve number of kills
sub getKills
{
    my $self = shift;
    if(!$self->{'kills'}) { $self->{'kills'} = 1; }
    return($self->{'kills'});
}

#------------------------------------------------------------------------------
# Set a player's name
sub setName
{
    my $self = shift;
    my $name = shift;

    $self->{'name'} = $name;
    $self->{'name_clean'} = $self->cleanName();
    $self->{'name_html'} = $self->htmlName($name);
    $self->{'previous_names'}{$self->{'name_html'}} += 1;
}

#------------------------------------------------------------------------------
# Record a player connect
sub connect
{
    my $self = shift;
    my $ip = shift;
    my $newtimestamp = shift;
    
    # Update the player's last IP address
    if(length($ip) >= 7)
    {
	    $self->{'last_ip'} = $ip;
    }

    # Update the player's connection count
    $self->{'connects'} += 1;

    # Update the last_date field if $newdatetime is indeed newer
    if ($newtimestamp > $self->{'last_date'} )
    {
        $self->{'last_date'} = $newtimestamp;
    }
}

#------------------------------------------------------------------------------
# Retrieve a player's IP address
sub getIP
{
    my $self = shift; 
    my $ip = $self->{'last_ip'};

    my $return;

    my($a,$b,$c,$d) = split(/\./, $ip);

    if(($a eq "192" && $b eq "168") || ($a eq "10"))
    {
	    $return = "$ip <b>LAN PLAYER</b>";
    }
    else
    {
	    $return = "<b>$ip</b>\n";
    }

    if(!$ip)
    {
	    $return = "<b>Unresolved</b>";
    }
    return($return);
}

#------------------------------------------------------------------------------
# Retrieve a player's last connection timestamp
sub getLastConnect
{
    my $self = shift; 
    if(!$self->{'last_date'}) { $self->{'last_date'} = 0; }
    return($self->{'last_date'});
}

#------------------------------------------------------------------------------
# Add time to player's total
sub addToTime
{
    my $self      = shift;
    my $time      = shift;
    my $addToTime = 0;

    if(($time - $self->{'start_time'}) > 43200)
    {
	    $addToTime = 1800;
    }
    else
    {
	    $addToTime = $time - $self->{'start_time'}
    }

    $self->{'total_time'}  += $addToTime;
    $self->{'start_time'}   = 0;
    $self->{'disconnects'} += 1;
}

#------------------------------------------------------------------------------
# Record player's start time
sub recordStartTime
{   
    my $self = shift;
    $self->{'start_time'} = shift;
}

#------------------------------------------------------------------------------
# Add to player's chatter counter
sub chatter
{
    my $self = shift;
    $self->{'chatter'} += 1;
}

#------------------------------------------------------------------------------
# Record a player's HTML-safe name
sub htmlName
{
    my $self = shift;
    my $name = $self->{'name'};

    $name =~ s/\</\&lt\;/g;
    $name =~ s/\>/\&gt\;/g;

    $self->{'name_html'} = $name;

    # rename colors
    return($name);
}

#------------------------------------------------------------------------------
# Clean a name of non-HTML and non-filename characters
sub cleanName
{
    my $self = shift;
    my $name = $self->{'name'};
    
    # rename non filesystem safe chars
    $name =~ s/ /_/g;
    $name =~ s/\=/\[EQUAL\]/g;
    $name =~ s/\,/\[CMA\]/g;
    $name =~ s/;/\[SMICLN\]/g;
    $name =~ s/:/\[CLN\]/g;
    $name =~ s/\"/\[DBLQT\]/g;
    $name =~ s/\'/\[SNGLQT\]/g;
    $name =~ s/\{/\[LB2\]/g;
    $name =~ s/\}/\[RB2\]/g;
    $name =~ s/\+/\[PLS\]/g;
    $name =~ s/\-/\[DSH\]/g;
    $name =~ s/\(/\[LPRN\]/g;
    $name =~ s/\)/\[RPRN\]/g;
    $name =~ s/\*/\[STR\]/g;
    $name =~ s/\&/\[AND\]/g;
    $name =~ s/\|/\[PIPE\]/g;
    $name =~ s/\\/\[BSLSH\]/g;
    $name =~ s/\//\[FSLSH\]/g;
    $name =~ s/\?/\[QUESTN\]/g;
    $name =~ s/\</\[LT\]/g;
    $name =~ s/\>/\[GT\]/g;
    $name =~ s/\./\[DOT\]/g;
    $name =~ s/\!/\[XCLM\]/g;
    $name =~ s/\@/\[AT\]/g;
    $name =~ s/\#/\[PND\]/g;
    $name =~ s/\$/\[DLLR\]/g;
    $name =~ s/\%/\[PRCNT\]/g;
    $name =~ s/\^/\[CRT\]/g;
    $name =~ s/\`/\[BTICK\]/g;
    if(length($name) > 100) { $name = substr($name, 0, 100); }      
    return($name);
}
###############################################################################

1;
